package middleware

import (
	"context"
	"net/http"
	"testing"
	"time"

	ctxUtil "gitlab.wanzhuangkj.com/tush/xpkg/gin/ctxUtils"
	"gitlab.wanzhuangkj.com/tush/xpkg/utils"

	"github.com/gin-gonic/gin"
	"github.com/stretchr/testify/assert"
)

func runRequestIDHTTPServer(fn func(c *gin.Context)) string {
	serverAddr, requestAddr := utils.GetLocalHTTPAddrPairs()

	gin.SetMode(gin.ReleaseMode)
	r := gin.Default()
	r.Use(RequestID(
	//WithRequestIDKey("my_req_id"),
	//WithHeaderRequestIDKey("My-X-Req-Id"),
	))
	r.GET("/ping", func(c *gin.Context) {
		fn(c)
		c.String(200, "pong")
	})

	go func() {
		err := r.Run(serverAddr)
		if err != nil {
			panic(err)
		}
	}()

	time.Sleep(time.Millisecond * 200)
	return requestAddr
}

func TestFieldRequestIDFromContext(t *testing.T) {
	requestAddr := runRequestIDHTTPServer(func(c *gin.Context) {
		str := ctxUtil.CtxRequestID(c)
		t.Log(str)
		field := ctxUtil.CtxTraceIDField(c)
		t.Log(field)

		str = HeaderRequestID(c)
		t.Log(str)
		field = HeaderRequestIDField(c)
		t.Log(field)

		str = ctxUtil.CtxRequestID(c)
		t.Log(str)
		field = ctxUtil.CtxTraceIDField(c)
		t.Log(field)

		c.Set("foo", "bar")

		ctx := WrapCtx(c)
		t.Log(ctx.Value(ctxUtil.ContextTraceIDKey))
		t.Log(GetFromCtx(ctx, "foo"))
		t.Log(ctxUtil.CtxTraceIDField(ctx))
		t.Log(GetFromCtx(ctx, "not-exist"))
		t.Log(GetFromHeader(ctx, ctxUtil.HeaderXRequestIDKey))
		t.Log(GetFromHeader(ctx, "Accept"))
		t.Log(GetFromHeader(ctx, "not-exist"))
		t.Log(GetFromHeaders(ctx, "Accept"))
		t.Log(GetFromHeaders(ctx, "not-exist"))

		cctx := c
		c2, ctx2 := AdaptCtx(cctx)
		t.Log(ctxUtil.CtxRequestID(c2))
		t.Log(ctx2.Value(ctxUtil.ContextTraceIDKey))
	})

	_, err := http.Get(requestAddr + "/ping")
	assert.NoError(t, err)

	defer func() { recover() }()
	req, _ := http.NewRequest("GET", requestAddr+"/ping", nil)
	req.Header.Add("Accept", "application/json")
	req.Header.Add("Accept", "text/html")
	req.Header.Set(ctxUtil.HeaderXRequestIDKey, "2ab996de-cc03-412d-ba0a-79596efa6947")
	resp, err := http.DefaultClient.Do(req)
	assert.NoError(t, err)
	defer resp.Body.Close()
}

func TestGetRequestIDFromContext(t *testing.T) {
	str := ctxUtil.CtxRequestID(&gin.Context{})
	assert.Equal(t, "", str)
	str = ctxUtil.CtxRequestID(context.Background())
	assert.Equal(t, "", str)
}

func TestRequestIDKeyOptions(t *testing.T) {
	opts := []RequestIDOption{
		WithContextRequestIDKey("xx"), // invalid settings
		WithContextRequestIDKey("my_req_id"),
		WithHeaderRequestIDKey("xx"), // invalid settings
		WithHeaderRequestIDKey("My-X-Req-Id"),
	}

	o := defaultRequestIDOptions()
	o.apply(opts...)
	o.setRequestIDKey()

	t.Log(ctxUtil.ContextTraceIDKey, ctxUtil.HeaderXRequestIDKey)

	assert.Equal(t, "my_req_id", ctxUtil.ContextTraceIDKey)
	assert.Equal(t, "My-X-Req-Id", ctxUtil.HeaderXRequestIDKey)
}
